def info_print(message):
    """
    Function which print information message

    :param message: message
    """
    print(f"[INFO] {message}", end="\n\n")


def warning_print(message):
    """
    Function which print warning message

    :param message: message
    """
    print(f"[WARNING] {message}", end="\n\n")


def error_print(message):
    """
    Function which print error message

    :param message: message
    """
    print(f"[ERROR] {message}", end="\n\n")
